RequireSystemScript('time.js');

TRANSPARENT = CreateColor(0,0,0,0);

font = GetSystemFont();

function game() {
	font.drawText(220, 220, "Loading the Sphere...");
	FlipScreen();
	Delay(3000);// wait screen switch
	sf = new StarField(GetScreenWidth(), GetScreenHeight(), 400, new FlyingStarFieldEngine());
	loop(sf);
}


function loop(runnable) {

	delay = 40;
	next = GetTime() + delay;

	while (!IsAnyKeyPressed()) {
		runnable.run();
		while (GetTime() < next) { }
		next += delay;
	}

}


function FlyingStarFieldEngine() {

	this.initStar = function(star, field) {
		star.x = Math.random() * field.w;
		star.y = Math.random() * field.h;
		star.dx = -Math.random() * 4 - 0.5;
	}
	
	this.reinitStar = function(star, field) {
		star.x = field.w - 1;
		star.y = Math.random() * field.h;
		star.dx = -Math.random() * 4 - 0.5;
	}
	

}


function StarField(w, h, c, engine) {
	this.stars = new Array(c);
	this.flares = new Array();
	this.w = w;
	this.h = h;
	this.hh = this.h / 2;
	this.hw = this.w / 2;
	this.dv = this.hh * this.hh;
	this.engine = engine;
	this.p = 0;
	this.flares[0] = CreateStarImage(1, CreateColor(0, 255, 0, 255));
	this.flares[1] = CreateStarImage(2, CreateColor(128, 255, 128, 255));
	this.flares[2] = CreateStarImage(1, CreateColor(192, 255, 192, 255));
	this.flares[3] = CreateStarImage(2, CreateColor(128, 255,128, 255));
	for (i = 0; i < c; i++) {
		this.stars[i] = new Star(0, 0, this.flares[Math.floor(Math.random() * this.flares.length)]);
		this.engine.initStar(this.stars[i], this);
	}

	
	this.drawNormal = function() {
		for (i = 0; i < this.stars.length; i++) {
			star = this.stars[i];
			nx = star.x - this.hw;
			ny = star.y - this.hh;
			d = Math.sqrt(nx * nx + ny * ny) / this.hh;
			//d = Math.sqrt(d);
			if (this.p < 1) {
				d *= -0.45;
				nx = (nx + nx * d) * this.p + this.hw;
				ny = (ny + ny * d) * this.p + this.hh;

			} else {
				d *= this.p - 1.45;
				nx = nx + nx * d + this.hw;
				ny = ny + ny * d + this.hh;
			}
			
			star.flare.blit(nx - star.r, ny - star.r);
		}
		if (this.p < 2) {
			if (this.p < 0.7) {
				font.drawText(220, 220, "Loading the Sphere...");
				this.p += 0.005;
			} else if (this.p < 1.5) {
				font.drawText(220, 220, "Entering the Sphere...");
				this.p += 0.005 + (this.p - 0.7) / 20;
			} else {
				this.p += 0.02;
			}
		} else {
			font.drawText(220, 220, "Welcome");
		}
		
	}
	
	this.sign = function(s) {
		if (s > 0) return 1;
		if (s < 0) return -1;
		return 0;
	}
	
	this.move = function() {
		for (i = 0; i < this.stars.length; i++) {
			star = this.stars[i];
			star.x += star.dx;
			star.y += star.dy;
			if (star.x < 0 || star.x > this.w || star.y <= 0 || star.y >= this.h) {
				this.engine.reinitStar(star, this);
			}
		}
	
	}
	
	this.run = function() {
		this.move();
		this.drawNormal();
		FlipScreen();
	}
}

function Star(x, y, flare) {
	this.x = x;
	this.y = y;
	this.dx = 0;
	this.dy = 0;
	this.r = (flare.width - 1) / 2;
	this.flare = flare;
}

function CreateStarImage(r, col) {

	s = r * 2 + 1;
	surf = CreateSurface(s, s, TRANSPARENT);
	c = r + 1;
	for (y = 0; y < s; y++) {
		dy = y - c;
		for (x = 0; x < s; x++) {
			dx = x - c;
			rp = Math.sqrt(dx * dx + dy * dy)
			
			if (rp < r) {			
				a = Math.pow(1 - rp / r, 2) * 255;
				col.alpha = a;
				surf.setPixel(x, y, col);
			} else {
				surf.setPixel(x, y, TRANSPARENT);
			}
			
		}
	}
	return surf.createImage();
}

